@@ -71,6 +71,7 @@ enum port_ops {
 };
 
 static struct ibmad_port *srcport;
+static struct ibmad_ports_pair *srcports;
 static uint64_t speed; /* no state change */
 static uint64_t espeed; /* no state change */
 static uint64_t fdr10; /* no state change */
@@ -461,14 +462,18 @@ int main(int argc, char **argv)
 	if (argc < 2)
 		ibdiag_show_usage();
 
-	srcport = mad_rpc_open_port(ibd_ca, ibd_ca_port, mgmt_classes, 3);
+	srcports = mad_rpc_open_port2(ibd_ca, ibd_ca_port, mgmt_classes, 3, 1);
+	if (!srcports)
+		IBEXIT("Failed to open '%s' port '%d'", ibd_ca, ibd_ca_port);
+
+	srcport = srcports->smi.port;
 	if (!srcport)
 		IBEXIT("Failed to open '%s' port '%d'", ibd_ca, ibd_ca_port);
 
 	smp_mkey_set(srcport, ibd_mkey);
 
-	if (resolve_portid_str(ibd_ca, ibd_ca_port, &portid, argv[0],
-			       ibd_dest_type, ibd_sm_id, srcport) < 0)
+	if (resolve_portid_str(srcports->gsi.ca_name, ibd_ca_port, &portid, argv[0],
+			       ibd_dest_type, ibd_sm_id, srcports->gsi.port) < 0)
 		IBEXIT("can't resolve destination port %s", argv[0]);
 
 	if (argc > 1)
@@ -810,6 +815,6 @@ int main(int argc, char **argv)
 	}
 
 close_port:
-	mad_rpc_close_port(srcport);
+	mad_rpc_close_port2(srcports);
 	exit(0);
 }
