@@ -4,7 +4,7 @@
 #define _GNU_SOURCE
 #include <stdio.h>
 #include <stdlib.h>
-#include <string.h>
+#include <libgen.h>
 #include <stdbool.h>
 #include <errno.h>
 #include <unistd.h>
@@ -27,6 +27,7 @@
  * NAME_PCI - based on PCI/slot/function location
  * NAME_GUID - based on node GUID
  * NAME_ONBOARD - based on-board device index
+ * NAME_FIXED - rename the device to the fixed named in the next argument
  *
  * The stable names are combination of device type technology and rename mode.
  * Infiniband - ib*
@@ -56,6 +57,7 @@ struct data {
 	uint64_t node_guid;
 	char *name;
 	int idx;
+	int name_assign_type;
 };
 
 static bool debug_mode;
@@ -484,6 +486,19 @@ out:
 	return 0;
 }
 
+static int set_fixed_name(struct data *d, char *name)
+{
+	int ret;
+
+	ret = asprintf(&d->name, "%s", name);
+	if (ret == -1) {
+		d->name = NULL;
+		return -ENOMEM;
+	}
+
+	return 0;
+}
+
 static int device_rename(struct nl_sock *nl, struct data *d)
 {
 	struct nlmsghdr *hdr;
@@ -541,6 +556,11 @@ static int get_nldata_cb(struct nl_msg *msg, void *data)
 
 	d->idx = nla_get_u32(tb[RDMA_NLDEV_ATTR_DEV_INDEX]);
 	d->node_guid = nla_get_u64(tb[RDMA_NLDEV_ATTR_NODE_GUID]);
+
+	if (tb[RDMA_NLDEV_ATTR_NAME_ASSIGN_TYPE])
+		d->name_assign_type =
+			nla_get_u8(tb[RDMA_NLDEV_ATTR_NAME_ASSIGN_TYPE]);
+
 	return NL_STOP;
 }
 
@@ -549,6 +569,7 @@ enum name_policy {
 	NAME_PCI = 1 << 1,
 	NAME_GUID = 1 << 2,
 	NAME_ONBOARD = 1 << 3,
+	NAME_FIXED = 1 << 4,
 	NAME_ERROR = 1 << 8
 };
 
@@ -562,6 +583,8 @@ static int str2policy(const char *np)
 		return NAME_GUID;
 	if (!strcmp(np, "NAME_ONBOARD"))
 		return NAME_ONBOARD;
+	if (!strcmp(np, "NAME_FIXED"))
+		return NAME_FIXED;
 	if (!strcmp(np, "NAME_FALLBACK"))
 		return NAME_ONBOARD | NAME_PCI;
 	return NAME_ERROR;
@@ -598,6 +621,11 @@ int main(int argc, char **argv)
 		goto err;
 	}
 
+	if (np & NAME_FIXED && argc < 3) {
+		pr_err("%s: No name specified\n", d.curr);
+		goto err;
+	}
+
 	pr_dbg("%s: Requested policy is %s\n", d.curr, argv[1]);
 
 	if (np & NAME_KERNEL) {
@@ -617,6 +645,13 @@ int main(int argc, char **argv)
 		goto out;
 	}
 
+	if (d.name_assign_type == RDMA_NAME_ASSIGN_TYPE_USER) {
+		pr_dbg("%s: Leave user-assigned names, do nothing\n", d.curr);
+		/* Do nothing */
+		ret = 0;
+		goto out;
+	}
+
 	if (d.idx == -1 || !d.prefix) {
 		pr_err("%s: Failed to get current device name and index\n",
 		       d.curr);
@@ -630,6 +665,8 @@ int main(int argc, char **argv)
 		ret = by_pci(&d);
 	if (ret && (np & NAME_GUID))
 		ret = by_guid(&d);
+	if (ret && (np & NAME_FIXED))
+		ret = set_fixed_name(&d, argv[2]);
 	if (ret)
 		goto out;
 
