@@ -57,19 +57,25 @@ enum {
 
 static int page_size;
 static int use_odp;
+static int implicit_odp;
+static int prefetch_mr;
 static int use_ts;
 static int validate_buf;
+static int use_dm;
+static int use_new_send;
 
 struct pingpong_context {
 	struct ibv_context	*context;
 	struct ibv_comp_channel *channel;
 	struct ibv_pd		*pd;
 	struct ibv_mr		*mr;
+	struct ibv_dm		*dm;
 	union {
 		struct ibv_cq		*cq;
 		struct ibv_cq_ex	*cq_ex;
 	} cq_s;
 	struct ibv_qp		*qp;
+	struct ibv_qp_ex	*qpx;
 	char			*buf;
 	int			 size;
 	int			 send_flags;
@@ -369,21 +375,26 @@ static struct pingpong_context *pp_init_ctx(struct ibv_device *ib_dev, int size,
 		goto clean_comp_channel;
 	}
 
-	if (use_odp || use_ts) {
+	if (use_odp || use_ts || use_dm) {
 		const uint32_t rc_caps_mask = IBV_ODP_SUPPORT_SEND |
 					      IBV_ODP_SUPPORT_RECV;
 		struct ibv_device_attr_ex attrx;
 
 		if (ibv_query_device_ex(ctx->context, NULL, &attrx)) {
 			fprintf(stderr, "Couldn't query device for its features\n");
-			goto clean_comp_channel;
+			goto clean_pd;
 		}
 
 		if (use_odp) {
 			if (!(attrx.odp_caps.general_caps & IBV_ODP_SUPPORT) ||
 			    (attrx.odp_caps.per_transport_caps.rc_odp_caps & rc_caps_mask) != rc_caps_mask) {
 				fprintf(stderr, "The device isn't ODP capable or does not support RC send and receive with ODP\n");
-				goto clean_comp_channel;
+				goto clean_pd;
+			}
+			if (implicit_odp &&
+			    !(attrx.odp_caps.general_caps & IBV_ODP_SUPPORT_IMPLICIT)) {
+				fprintf(stderr, "The device doesn't support implicit ODP\n");
+				goto clean_pd;
 			}
 			access_flags |= IBV_ACCESS_ON_DEMAND;
 		}
@@ -391,16 +402,62 @@ static struct pingpong_context *pp_init_ctx(struct ibv_device *ib_dev, int size,
 		if (use_ts) {
 			if (!attrx.completion_timestamp_mask) {
 				fprintf(stderr, "The device isn't completion timestamp capable\n");
-				goto clean_comp_channel;
+				goto clean_pd;
 			}
 			ctx->completion_timestamp_mask = attrx.completion_timestamp_mask;
 		}
+
+		if (use_dm) {
+			struct ibv_alloc_dm_attr dm_attr = {};
+
+			if (!attrx.max_dm_size) {
+				fprintf(stderr, "Device doesn't support dm allocation\n");
+				goto clean_pd;
+			}
+
+			if (attrx.max_dm_size < size) {
+				fprintf(stderr, "Device memory is insufficient\n");
+				goto clean_pd;
+			}
+
+			dm_attr.length = size;
+			ctx->dm = ibv_alloc_dm(ctx->context, &dm_attr);
+			if (!ctx->dm) {
+				fprintf(stderr, "Dev mem allocation failed\n");
+				goto clean_pd;
+			}
+
+			access_flags |= IBV_ACCESS_ZERO_BASED;
+		}
+	}
+
+	if (implicit_odp) {
+		ctx->mr = ibv_reg_mr(ctx->pd, NULL, SIZE_MAX, access_flags);
+	} else {
+		ctx->mr = use_dm ? ibv_reg_dm_mr(ctx->pd, ctx->dm, 0,
+						 size, access_flags) :
+			ibv_reg_mr(ctx->pd, ctx->buf, size, access_flags);
 	}
-	ctx->mr = ibv_reg_mr(ctx->pd, ctx->buf, size, access_flags);
 
 	if (!ctx->mr) {
 		fprintf(stderr, "Couldn't register MR\n");
-		goto clean_pd;
+		goto clean_dm;
+	}
+
+	if (prefetch_mr) {
+		struct ibv_sge sg_list;
+		int ret;
+
+		sg_list.lkey = ctx->mr->lkey;
+		sg_list.addr = (uintptr_t)ctx->buf;
+		sg_list.length = size;
+
+		ret = ibv_advise_mr(ctx->pd, IBV_ADVISE_MR_ADVICE_PREFETCH_WRITE,
+				    IB_UVERBS_ADVISE_MR_FLAG_FLUSH,
+				    &sg_list, 1);
+
+		if (ret)
+			fprintf(stderr, "Couldn't prefetch MR(%d). Continue anyway\n", ret);
 	}
 
 	if (use_ts) {
@@ -437,16 +494,38 @@ static struct pingpong_context *pp_init_ctx(struct ibv_device *ib_dev, int size,
 			.qp_type = IBV_QPT_RC
 		};
 
-		ctx->qp = ibv_create_qp(ctx->pd, &init_attr);
+		if (use_new_send) {
+			struct ibv_qp_init_attr_ex init_attr_ex = {};
+
+			init_attr_ex.send_cq = pp_cq(ctx);
+			init_attr_ex.recv_cq = pp_cq(ctx);
+			init_attr_ex.cap.max_send_wr = 1;
+			init_attr_ex.cap.max_recv_wr = rx_depth;
+			init_attr_ex.cap.max_send_sge = 1;
+			init_attr_ex.cap.max_recv_sge = 1;
+			init_attr_ex.qp_type = IBV_QPT_RC;
+
+			init_attr_ex.comp_mask |= IBV_QP_INIT_ATTR_PD |
+						  IBV_QP_INIT_ATTR_SEND_OPS_FLAGS;
+			init_attr_ex.pd = ctx->pd;
+			init_attr_ex.send_ops_flags = IBV_QP_EX_WITH_SEND;
+
+			ctx->qp = ibv_create_qp_ex(ctx->context, &init_attr_ex);
+		} else {
+			ctx->qp = ibv_create_qp(ctx->pd, &init_attr);
+		}
+
 		if (!ctx->qp)  {
 			fprintf(stderr, "Couldn't create QP\n");
 			goto clean_cq;
 		}
 
+		if (use_new_send)
+			ctx->qpx = ibv_qp_to_qp_ex(ctx->qp);
+
 		ibv_query_qp(ctx->qp, &attr, IBV_QP_CAP, &init_attr);
-		if (init_attr.cap.max_inline_data >= size) {
+		if (init_attr.cap.max_inline_data >= size && !use_dm)
 			ctx->send_flags |= IBV_SEND_INLINE;
-		}
 	}
 
 	{
@@ -478,6 +557,10 @@ clean_cq:
 clean_mr:
 	ibv_dereg_mr(ctx->mr);
 
+clean_dm:
+	if (ctx->dm)
+		ibv_free_dm(ctx->dm);
+
 clean_pd:
 	ibv_dealloc_pd(ctx->pd);
 
@@ -514,6 +597,13 @@ static int pp_close_ctx(struct pingpong_context *ctx)
 		return 1;
 	}
 
+	if (ctx->dm) {
+		if (ibv_free_dm(ctx->dm)) {
+			fprintf(stderr, "Couldn't free DM\n");
+			return 1;
+		}
+	}
+
 	if (ibv_dealloc_pd(ctx->pd)) {
 		fprintf(stderr, "Couldn't deallocate PD\n");
 		return 1;
@@ -540,7 +630,7 @@ static int pp_close_ctx(struct pingpong_context *ctx)
 static int pp_post_recv(struct pingpong_context *ctx, int n)
 {
 	struct ibv_sge list = {
-		.addr	= (uintptr_t) ctx->buf,
+		.addr	= use_dm ? 0 : (uintptr_t) ctx->buf,
 		.length = ctx->size,
 		.lkey	= ctx->mr->lkey
 	};
@@ -562,7 +652,7 @@ static int pp_post_recv(struct pingpong_context *ctx, int n)
 static int pp_post_send(struct pingpong_context *ctx)
 {
 	struct ibv_sge list = {
-		.addr	= (uintptr_t) ctx->buf,
+		.addr	= use_dm ? 0 : (uintptr_t) ctx->buf,
 		.length = ctx->size,
 		.lkey	= ctx->mr->lkey
 	};
@@ -575,7 +665,19 @@ static int pp_post_send(struct pingpong_context *ctx)
 	};
 	struct ibv_send_wr *bad_wr;
 
-	return ibv_post_send(ctx->qp, &wr, &bad_wr);
+	if (use_new_send) {
+		ibv_wr_start(ctx->qpx);
+
+		ctx->qpx->wr_id = PINGPONG_SEND_WRID;
+		ctx->qpx->wr_flags = ctx->send_flags;
+
+		ibv_wr_send(ctx->qpx);
+		ibv_wr_set_sge(ctx->qpx, list.lkey, list.addr, list.length);
+
+		return ibv_wr_complete(ctx->qpx);
+	} else {
+		return ibv_post_send(ctx->qp, &wr, &bad_wr);
+	}
 }
 
 struct ts_params {
@@ -679,8 +781,12 @@ static void usage(const char *argv0)
 	printf("  -e, --events           sleep on CQ events (default poll)\n");
 	printf("  -g, --gid-idx=<gid index> local port gid index\n");
 	printf("  -o, --odp		    use on demand paging\n");
+	printf("  -O, --iodp		    use implicit on demand paging\n");
+	printf("  -P, --prefetch	    prefetch an ODP MR\n");
 	printf("  -t, --ts	            get CQE with timestamp\n");
 	printf("  -c, --chk	            validate received buffer\n");
+	printf("  -j, --dm	            use device memory\n");
+	printf("  -N, --new_send            use new post send WR API\n");
 }
 
 int main(int argc, char *argv[])
@@ -725,12 +831,16 @@ int main(int argc, char *argv[])
 			{ .name = "events",   .has_arg = 0, .val = 'e' },
 			{ .name = "gid-idx",  .has_arg = 1, .val = 'g' },
 			{ .name = "odp",      .has_arg = 0, .val = 'o' },
+			{ .name = "iodp",     .has_arg = 0, .val = 'O' },
+			{ .name = "prefetch", .has_arg = 0, .val = 'P' },
 			{ .name = "ts",       .has_arg = 0, .val = 't' },
 			{ .name = "chk",      .has_arg = 0, .val = 'c' },
+			{ .name = "dm",       .has_arg = 0, .val = 'j' },
+			{ .name = "new_send", .has_arg = 0, .val = 'N' },
 			{}
 		};
 
-		c = getopt_long(argc, argv, "p:d:i:s:m:r:n:l:eg:otc",
+		c = getopt_long(argc, argv, "p:d:i:s:m:r:n:l:eg:oOPtcjN",
 				long_options, NULL);
 
 		if (c == -1)
@@ -792,6 +902,13 @@ int main(int argc, char *argv[])
 		case 'o':
 			use_odp = 1;
 			break;
+		case 'P':
+			prefetch_mr = 1;
+			break;
+		case 'O':
+			use_odp = 1;
+			implicit_odp = 1;
+			break;
 		case 't':
 			use_ts = 1;
 			break;
@@ -799,6 +916,14 @@ int main(int argc, char *argv[])
 			validate_buf = 1;
 			break;
 
+		case 'j':
+			use_dm = 1;
+			break;
+
+		case 'N':
+			use_new_send = 1;
+			break;
+
 		default:
 			usage(argv[0]);
 			return 1;
@@ -812,6 +937,16 @@ int main(int argc, char *argv[])
 		return 1;
 	}
 
+	if (use_odp && use_dm) {
+		fprintf(stderr, "DM memory region can't be on demand\n");
+		return 1;
+	}
+
+	if (!use_odp && prefetch_mr) {
+		fprintf(stderr, "prefetch is valid only with on-demand memory region\n");
+		return 1;
+	}
+
 	if (use_ts) {
 		ts.comp_recv_max_time_delta = 0;
 		ts.comp_recv_min_time_delta = 0xffffffff;
@@ -915,6 +1050,13 @@ int main(int argc, char *argv[])
 		if (validate_buf)
 			for (int i = 0; i < size; i += page_size)
 				ctx->buf[i] = i / page_size % sizeof(char);
+
+		if (use_dm)
+			if (ibv_memcpy_to_dm(ctx->dm, 0, (void *)ctx->buf, size)) {
+				fprintf(stderr, "Copy to dm buffer failed\n");
+				return 1;
+			}
+
 		if (pp_post_send(ctx)) {
 			fprintf(stderr, "Couldn't post send\n");
 			return 1;
@@ -1038,6 +1180,11 @@ int main(int argc, char *argv[])
 		}
 
 		if ((!servername) && (validate_buf)) {
+			if (use_dm)
+				if (ibv_memcpy_from_dm(ctx->buf, ctx->dm, 0, size)) {
+					fprintf(stderr, "Copy from DM buffer failed\n");
+					return 1;
+				}
 			for (int i = 0; i < size; i += page_size)
 				if (ctx->buf[i] != i / page_size % sizeof(char))
 					printf("invalid data in page %d\n",
