@@ -45,6 +45,7 @@
 #include <inttypes.h>
 
 #include "ibverbs.h"
+#include "util/rdma_nl.h"
 
 struct ibv_mem_node {
 	enum {
@@ -133,6 +134,9 @@ int ibv_fork_init(void)
 	if (mm_root)
 		return 0;
 
+	if (ibv_is_fork_initialized() == IBV_FORK_UNNEEDED)
+		return 0;
+
 	if (too_late)
 		return EINVAL;
 
@@ -174,6 +178,14 @@ int ibv_fork_init(void)
 	return 0;
 }
 
+enum ibv_fork_status ibv_is_fork_initialized(void)
+{
+	if (get_copy_on_fork())
+		return IBV_FORK_UNNEEDED;
+
+	return mm_root ? IBV_FORK_ENABLED : IBV_FORK_DISABLED;
+}
+
 static struct ibv_mem_node *__mm_prev(struct ibv_mem_node *node)
 {
 	if (node->left) {
