@@ -33,6 +33,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <math.h>
 #include <errno.h>
 #include <pthread.h>
 #include <sys/mman.h>
@@ -42,6 +43,33 @@
 #include "hns_roce_u_db.h"
 #include "hns_roce_u_hw_v2.h"
 
+static bool hns_roce_whether_need_lock(struct ibv_pd *pd)
+{
+	struct hns_roce_pad *pad = to_hr_pad(pd);
+
+	return !(pad && pad->td);
+}
+
+static int hns_roce_spinlock_init(struct hns_roce_spinlock *hr_lock,
+				  bool need_lock)
+{
+	hr_lock->need_lock = need_lock;
+
+	if (need_lock)
+		return pthread_spin_init(&hr_lock->lock,
+					 PTHREAD_PROCESS_PRIVATE);
+
+	return 0;
+}
+
+static int hns_roce_spinlock_destroy(struct hns_roce_spinlock *hr_lock)
+{
+	if (hr_lock->need_lock)
+		return pthread_spin_destroy(&hr_lock->lock);
+
+	return 0;
+}
+
 void hns_roce_init_qp_indices(struct hns_roce_qp *qp)
 {
 	qp->sq.head = 0;
@@ -85,38 +113,137 @@ int hns_roce_u_query_port(struct ibv_context *context, uint8_t port,
 	return ibv_cmd_query_port(context, port, attr, &cmd, sizeof(cmd));
 }
 
-struct ibv_pd *hns_roce_u_alloc_pd(struct ibv_context *context)
+struct ibv_td *hns_roce_u_alloc_td(struct ibv_context *context,
+				   struct ibv_td_init_attr *attr)
 {
-	struct ibv_alloc_pd cmd;
-	struct hns_roce_pd *pd;
-	struct hns_roce_alloc_pd_resp resp = {};
+	struct hns_roce_td *td;
 
-	pd = malloc(sizeof(*pd));
-	if (!pd)
-		return NULL;
-
-	if (ibv_cmd_alloc_pd(context, &pd->ibv_pd, &cmd, sizeof(cmd),
-			     &resp.ibv_resp, sizeof(resp))) {
-		free(pd);
+	if (attr->comp_mask) {
+		errno = EOPNOTSUPP;
 		return NULL;
 	}
 
+	td = calloc(1, sizeof(*td));
+	if (!td) {
+		errno = ENOMEM;
+		return NULL;
+	}
+
+	td->ibv_td.context = context;
+	atomic_init(&td->refcount, 1);
+
+	return &td->ibv_td;
+}
+
+int hns_roce_u_dealloc_td(struct ibv_td *ibv_td)
+{
+	struct hns_roce_td *td;
+
+	td = to_hr_td(ibv_td);
+	if (atomic_load(&td->refcount) > 1)
+		return EBUSY;
+
+	free(td);
+
+	return 0;
+}
+
+struct ibv_pd *hns_roce_u_alloc_pd(struct ibv_context *context)
+{
+	struct hns_roce_alloc_pd_resp resp = {};
+	struct ibv_alloc_pd cmd;
+	struct hns_roce_pd *pd;
+
+	pd = calloc(1, sizeof(*pd));
+	if (!pd) {
+		errno = ENOMEM;
+		return NULL;
+	}
+	errno = ibv_cmd_alloc_pd(context, &pd->ibv_pd, &cmd, sizeof(cmd),
+				 &resp.ibv_resp, sizeof(resp));
+	if (errno)
+		goto err;
+
+	atomic_init(&pd->refcount, 1);
 	pd->pdn = resp.pdn;
 
 	return &pd->ibv_pd;
+
+err:
+	free(pd);
+	return NULL;
 }
 
-int hns_roce_u_free_pd(struct ibv_pd *pd)
+struct ibv_pd *hns_roce_u_alloc_pad(struct ibv_context *context,
+				    struct ibv_parent_domain_init_attr *attr)
+{
+	struct hns_roce_pad *pad;
+
+	if (ibv_check_alloc_parent_domain(attr))
+		return NULL;
+
+	if (attr->comp_mask) {
+		errno = EOPNOTSUPP;
+		return NULL;
+	}
+
+	pad = calloc(1, sizeof(*pad));
+	if (!pad) {
+		errno = ENOMEM;
+		return NULL;
+	}
+
+	if (attr->td) {
+		pad->td = to_hr_td(attr->td);
+		atomic_fetch_add(&pad->td->refcount, 1);
+	}
+
+	pad->pd.protection_domain = to_hr_pd(attr->pd);
+	atomic_fetch_add(&pad->pd.protection_domain->refcount, 1);
+
+	atomic_init(&pad->pd.refcount, 1);
+	ibv_initialize_parent_domain(&pad->pd.ibv_pd,
+				     &pad->pd.protection_domain->ibv_pd);
+
+	return &pad->pd.ibv_pd;
+}
+
+static void hns_roce_free_pad(struct hns_roce_pad *pad)
+{
+	atomic_fetch_sub(&pad->pd.protection_domain->refcount, 1);
+
+	if (pad->td)
+		atomic_fetch_sub(&pad->td->refcount, 1);
+
+	free(pad);
+}
+
+static int hns_roce_free_pd(struct hns_roce_pd *pd)
 {
 	int ret;
 
-	ret = ibv_cmd_dealloc_pd(pd);
+	if (atomic_load(&pd->refcount) > 1)
+		return EBUSY;
+
+	ret = ibv_cmd_dealloc_pd(&pd->ibv_pd);
 	if (ret)
 		return ret;
 
-	free(to_hr_pd(pd));
+	free(pd);
+	return 0;
+}
 
-	return ret;
+int hns_roce_u_dealloc_pd(struct ibv_pd *ibv_pd)
+{
+	struct hns_roce_pad *pad = to_hr_pad(ibv_pd);
+	struct hns_roce_pd *pd = to_hr_pd(ibv_pd);
+
+	if (pad) {
+		hns_roce_free_pad(pad);
+		return 0;
+	}
+
+	return hns_roce_free_pd(pd);
 }
 
 struct ibv_xrcd *hns_roce_u_open_xrcd(struct ibv_context *context,
@@ -275,6 +402,11 @@ int hns_roce_u_dealloc_mw(struct ibv_mw *mw)
 	return 0;
 }
 
+enum {
+	CREATE_CQ_SUPPORTED_COMP_MASK = IBV_CQ_INIT_ATTR_MASK_FLAGS |
+					IBV_CQ_INIT_ATTR_MASK_PD,
+};
+
 enum {
 	CREATE_CQ_SUPPORTED_WC_FLAGS = IBV_WC_STANDARD_FLAGS |
 				       IBV_WC_EX_WITH_CVLAN,
@@ -283,14 +415,33 @@ enum {
 static int verify_cq_create_attr(struct ibv_cq_init_attr_ex *attr,
 				 struct hns_roce_context *context)
 {
-	if (!attr->cqe || attr->cqe > context->max_cqe)
+	struct hns_roce_pad *pad = to_hr_pad(attr->parent_domain);
+
+	if (!attr->cqe || attr->cqe > context->max_cqe) {
+		verbs_err(&context->ibv_ctx, "unsupported cq depth %u.\n",
+			  attr->cqe);
 		return EINVAL;
+	}
 
-	if (attr->comp_mask)
+	if (!check_comp_mask(attr->comp_mask, CREATE_CQ_SUPPORTED_COMP_MASK)) {
+		verbs_err(&context->ibv_ctx, "unsupported cq comps 0x%x\n",
+			  attr->comp_mask);
 		return EOPNOTSUPP;
+	}
 
-	if (!check_comp_mask(attr->wc_flags, CREATE_CQ_SUPPORTED_WC_FLAGS))
+	if (!check_comp_mask(attr->wc_flags, CREATE_CQ_SUPPORTED_WC_FLAGS)) {
+		verbs_err(&context->ibv_ctx, "unsupported wc flags 0x%llx.\n",
+			  attr->wc_flags);
 		return EOPNOTSUPP;
+	}
+
+	if (attr->comp_mask & IBV_CQ_INIT_ATTR_MASK_PD) {
+		if (!pad) {
+			verbs_err(&context->ibv_ctx, "failed to check the pad of cq.\n");
+			return EINVAL;
+		}
+		atomic_fetch_add(&pad->pd.refcount, 1);
+	}
 
 	attr->cqe = max_t(uint32_t, HNS_ROCE_MIN_CQE_NUM,
 			  roundup_pow_of_two(attr->cqe));
@@ -298,6 +449,22 @@ static int verify_cq_create_attr(struct ibv_cq_init_attr_ex *attr,
 	return 0;
 }
 
+static int hns_roce_cq_spinlock_init(struct hns_roce_cq *cq,
+				     struct ibv_cq_init_attr_ex *attr)
+{
+	bool need_lock = hns_roce_whether_need_lock(attr->parent_domain);
+
+	return hns_roce_spinlock_init(&cq->hr_lock, need_lock);
+}
+
+static int hns_roce_srq_spinlock_init(struct hns_roce_srq *srq,
+				      struct ibv_srq_init_attr_ex *attr)
+{
+	bool need_lock = hns_roce_whether_need_lock(attr->pd);
+
+	return hns_roce_spinlock_init(&srq->hr_lock, need_lock);
+}
+
 static int hns_roce_alloc_cq_buf(struct hns_roce_cq *cq)
 {
 	int buf_size = hr_hw_page_align(cq->cq_depth * cq->cqe_size);
@@ -328,8 +495,11 @@ static int exec_cq_create_cmd(struct ibv_context *context,
 	ret = ibv_cmd_create_cq_ex(context, attr, &cq->verbs_cq,
 				   &cmd_ex.ibv_cmd, sizeof(cmd_ex),
 				   &resp_ex.ibv_resp, sizeof(resp_ex), 0);
-	if (ret)
+	if (ret) {
+		verbs_err(verbs_get_ctx(context),
+			  "failed to exec create cq cmd, ret = %d.\n", ret);
 		return ret;
+	}
 
 	cq->cqn = resp_drv->cqn;
 	cq->flags = resp_drv->cap_flags;
@@ -354,7 +524,10 @@ static struct ibv_cq_ex *create_cq(struct ibv_context *context,
 		goto err;
 	}
 
-	ret = pthread_spin_init(&cq->lock, PTHREAD_PROCESS_PRIVATE);
+	if (attr->comp_mask & IBV_CQ_INIT_ATTR_MASK_PD)
+		cq->parent_domain = attr->parent_domain;
+
+	ret = hns_roce_cq_spinlock_init(cq, attr);
 	if (ret)
 		goto err_lock;
 
@@ -384,7 +557,7 @@ err_cmd:
 err_db:
 	hns_roce_free_buf(&cq->buf);
 err_buf:
-	pthread_spin_destroy(&cq->lock);
+	hns_roce_spinlock_destroy(&cq->hr_lock);
 err_lock:
 	free(cq);
 err:
@@ -437,6 +610,7 @@ int hns_roce_u_modify_cq(struct ibv_cq *cq, struct ibv_modify_cq_attr *attr)
 int hns_roce_u_destroy_cq(struct ibv_cq *cq)
 {
 	struct hns_roce_cq *hr_cq = to_hr_cq(cq);
+	struct hns_roce_pad *pad = to_hr_pad(hr_cq->parent_domain);
 	int ret;
 
 	ret = ibv_cmd_destroy_cq(cq);
@@ -447,7 +621,10 @@ int hns_roce_u_destroy_cq(struct ibv_cq *cq)
 			 HNS_ROCE_CQ_TYPE_DB);
 	hns_roce_free_buf(&hr_cq->buf);
 
-	pthread_spin_destroy(&hr_cq->lock);
+	hns_roce_spinlock_destroy(&hr_cq->hr_lock);
+
+	if (pad)
+		atomic_fetch_sub(&pad->pd.refcount, 1);
 
 	free(hr_cq);
 
@@ -508,13 +685,20 @@ static int verify_srq_create_attr(struct hns_roce_context *context,
 				  struct ibv_srq_init_attr_ex *attr)
 {
 	if (attr->srq_type != IBV_SRQT_BASIC &&
-	    attr->srq_type != IBV_SRQT_XRC)
+	    attr->srq_type != IBV_SRQT_XRC) {
+		verbs_err(&context->ibv_ctx,
+			  "unsupported srq type, type = %d.\n", attr->srq_type);
 		return -EINVAL;
+	}
 
 	if (!attr->attr.max_sge ||
 	    attr->attr.max_wr > context->max_srq_wr ||
-	    attr->attr.max_sge > context->max_srq_sge)
+	    attr->attr.max_sge > context->max_srq_sge) {
+		verbs_err(&context->ibv_ctx,
+			  "invalid srq attr size, max_wr = %u, max_sge = %u.\n",
+			  attr->attr.max_wr, attr->attr.max_sge);
 		return -EINVAL;
+	}
 
 	attr->attr.max_wr = max_t(uint32_t, attr->attr.max_wr,
 				  HNS_ROCE_MIN_SRQ_WQE_NUM);
@@ -631,8 +815,12 @@ static int exec_srq_create_cmd(struct ibv_context *context,
 	ret = ibv_cmd_create_srq_ex(context, &srq->verbs_srq, init_attr,
 				    &cmd_ex.ibv_cmd, sizeof(cmd_ex),
 				    &resp_ex.ibv_resp, sizeof(resp_ex));
-	if (ret)
+	if (ret) {
+		verbs_err(verbs_get_ctx(context),
+			  "failed to exec create srq cmd, ret = %d.\n",
+			  ret);
 		return ret;
+	}
 
 	srq->srqn = resp_ex.srqn;
 	srq->cap_flags = resp_ex.cap_flags;
@@ -644,6 +832,7 @@ static struct ibv_srq *create_srq(struct ibv_context *context,
 				  struct ibv_srq_init_attr_ex *init_attr)
 {
 	struct hns_roce_context *hr_ctx = to_hr_ctx(context);
+	struct hns_roce_pad *pad = to_hr_pad(init_attr->pd);
 	struct hns_roce_srq *srq;
 	int ret;
 
@@ -657,7 +846,10 @@ static struct ibv_srq *create_srq(struct ibv_context *context,
 		goto err;
 	}
 
-	if (pthread_spin_init(&srq->lock, PTHREAD_PROCESS_PRIVATE))
+	if (pad)
+		atomic_fetch_add(&pad->pd.refcount, 1);
+
+	if (hns_roce_srq_spinlock_init(srq, init_attr))
 		goto err_free_srq;
 
 	set_srq_param(context, srq, init_attr);
@@ -692,7 +884,7 @@ err_srq_buf:
 	free_srq_buf(srq);
 
 err_destroy_lock:
-	pthread_spin_destroy(&srq->lock);
+	hns_roce_spinlock_destroy(&srq->hr_lock);
 
 err_free_srq:
 	free(srq);
@@ -758,6 +950,7 @@ int hns_roce_u_query_srq(struct ibv_srq *srq, struct ibv_srq_attr *srq_attr)
 int hns_roce_u_destroy_srq(struct ibv_srq *ibv_srq)
 {
 	struct hns_roce_context *ctx = to_hr_ctx(ibv_srq->context);
+	struct hns_roce_pad *pad = to_hr_pad(ibv_srq->pd);
 	struct hns_roce_srq *srq = to_hr_srq(ibv_srq);
 	int ret;
 
@@ -770,7 +963,11 @@ int hns_roce_u_destroy_srq(struct ibv_srq *ibv_srq)
 	hns_roce_free_db(ctx, srq->rdb, HNS_ROCE_SRQ_TYPE_DB);
 	free_srq_buf(srq);
 
-	pthread_spin_destroy(&srq->lock);
+	hns_roce_spinlock_destroy(&srq->hr_lock);
+
+	if (pad)
+		atomic_fetch_sub(&pad->pd.refcount, 1);
+
 	free(srq);
 
 	return 0;
@@ -804,9 +1001,12 @@ static int check_qp_create_mask(struct hns_roce_context *ctx,
 				struct ibv_qp_init_attr_ex *attr)
 {
 	struct hns_roce_device *hr_dev = to_hr_dev(ctx->ibv_ctx.context.device);
+	int ret = 0;
 
-	if (!check_comp_mask(attr->comp_mask, CREATE_QP_SUP_COMP_MASK))
-		return EOPNOTSUPP;
+	if (!check_comp_mask(attr->comp_mask, CREATE_QP_SUP_COMP_MASK)) {
+		ret = EOPNOTSUPP;
+		goto out;
+	}
 
 	switch (attr->qp_type) {
 	case IBV_QPT_UD:
@@ -816,17 +1016,21 @@ static int check_qp_create_mask(struct hns_roce_context *ctx,
 	case IBV_QPT_RC:
 	case IBV_QPT_XRC_SEND:
 		if (!(attr->comp_mask & IBV_QP_INIT_ATTR_PD))
-			return EINVAL;
+			ret = EINVAL;
 		break;
 	case IBV_QPT_XRC_RECV:
 		if (!(attr->comp_mask & IBV_QP_INIT_ATTR_XRCD))
-			return EINVAL;
+			ret = EINVAL;
 		break;
 	default:
 		return EOPNOTSUPP;
 	}
 
-	return 0;
+out:
+	if (ret)
+		verbs_err(&ctx->ibv_ctx, "invalid comp_mask 0x%x.\n",
+			  attr->comp_mask);
+	return ret;
 }
 
 static int hns_roce_qp_has_rq(struct ibv_qp_init_attr_ex *attr)
@@ -851,8 +1055,13 @@ static int verify_qp_create_cap(struct hns_roce_context *ctx,
 	if (cap->max_send_wr > ctx->max_qp_wr ||
 	    cap->max_recv_wr > ctx->max_qp_wr ||
 	    cap->max_send_sge > ctx->max_sge  ||
-	    cap->max_recv_sge > ctx->max_sge)
+	    cap->max_recv_sge > ctx->max_sge) {
+		verbs_err(&ctx->ibv_ctx,
+			  "invalid qp cap size, max_send/recv_wr = {%u, %u}, max_send/recv_sge = {%u, %u}.\n",
+			  cap->max_send_wr, cap->max_recv_wr,
+			  cap->max_send_sge, cap->max_recv_sge);
 		return -EINVAL;
+	}
 
 	has_rq = hns_roce_qp_has_rq(attr);
 	if (!has_rq) {
@@ -861,12 +1070,20 @@ static int verify_qp_create_cap(struct hns_roce_context *ctx,
 	}
 
 	min_wqe_num = HNS_ROCE_V2_MIN_WQE_NUM;
-	if (cap->max_send_wr < min_wqe_num)
+	if (cap->max_send_wr < min_wqe_num) {
+		verbs_debug(&ctx->ibv_ctx,
+			    "change sq depth from %u to minimum %u.\n",
+			    cap->max_send_wr, min_wqe_num);
 		cap->max_send_wr = min_wqe_num;
+	}
 
 	if (cap->max_recv_wr) {
-		if (cap->max_recv_wr < min_wqe_num)
+		if (cap->max_recv_wr < min_wqe_num) {
+			verbs_debug(&ctx->ibv_ctx,
+				    "change rq depth from %u to minimum %u.\n",
+				    cap->max_recv_wr, min_wqe_num);
 			cap->max_recv_wr = min_wqe_num;
+		}
 
 		if (!cap->max_recv_sge)
 			return -EINVAL;
@@ -892,6 +1109,29 @@ static int verify_qp_create_attr(struct hns_roce_context *ctx,
 	return verify_qp_create_cap(ctx, attr);
 }
 
+static int hns_roce_qp_spinlock_init(struct ibv_qp_init_attr_ex *attr,
+				     struct hns_roce_qp *qp)
+{
+	bool need_lock = hns_roce_whether_need_lock(attr->pd);
+	int ret;
+
+	ret = hns_roce_spinlock_init(&qp->sq.hr_lock, need_lock);
+	if (ret)
+		return ret;
+
+	ret = hns_roce_spinlock_init(&qp->rq.hr_lock, need_lock);
+	if (ret)
+		hns_roce_spinlock_destroy(&qp->sq.hr_lock);
+
+	return ret;
+}
+
+void hns_roce_qp_spinlock_destroy(struct hns_roce_qp *qp)
+{
+	hns_roce_spinlock_destroy(&qp->rq.hr_lock);
+	hns_roce_spinlock_destroy(&qp->sq.hr_lock);
+}
+
 static int alloc_recv_rinl_buf(uint32_t max_sge,
 			       struct hns_roce_rinl_buf *rinl_buf)
 {
@@ -1246,6 +1486,11 @@ static int qp_exec_create_cmd(struct ibv_qp_init_attr_ex *attr,
 	ret = ibv_cmd_create_qp_ex2(&ctx->ibv_ctx.context, &qp->verbs_qp, attr,
 				    &cmd_ex.ibv_cmd, sizeof(cmd_ex),
 				    &resp_ex.ibv_resp, sizeof(resp_ex));
+	if (ret) {
+		verbs_err(&ctx->ibv_ctx,
+			  "failed to exec create qp cmd, ret = %d.\n", ret);
+		return ret;
+	}
 
 	qp->flags = resp_ex.drv_payload.cap_flags;
 	*dwqe_mmap_key = resp_ex.drv_payload.dwqe_mmap_key;
@@ -1282,8 +1527,6 @@ void hns_roce_free_qp_buf(struct hns_roce_qp *qp, struct hns_roce_context *ctx)
 {
 	qp_free_db(qp, ctx);
 	qp_free_wqe(qp);
-	pthread_spin_destroy(&qp->rq.lock);
-	pthread_spin_destroy(&qp->sq.lock);
 }
 
 static int hns_roce_alloc_qp_buf(struct ibv_qp_init_attr_ex *attr,
@@ -1292,14 +1535,6 @@ static int hns_roce_alloc_qp_buf(struct ibv_qp_init_attr_ex *attr,
 {
 	int ret;
 
-	ret = pthread_spin_init(&qp->sq.lock, PTHREAD_PROCESS_PRIVATE);
-	if (ret)
-		return ret;
-
-	ret = pthread_spin_init(&qp->rq.lock, PTHREAD_PROCESS_PRIVATE);
-	if (ret)
-		goto err_rq_lock;
-
 	ret = qp_alloc_wqe(&attr->cap, qp, ctx);
 	if (ret)
 		goto err_wqe;
@@ -1313,10 +1548,6 @@ static int hns_roce_alloc_qp_buf(struct ibv_qp_init_attr_ex *attr,
 err_db:
 	qp_free_wqe(qp);
 err_wqe:
-	pthread_spin_destroy(&qp->rq.lock);
-err_rq_lock:
-	pthread_spin_destroy(&qp->sq.lock);
-
 	return ret;
 }
 
@@ -1325,8 +1556,12 @@ static int mmap_dwqe(struct ibv_context *ibv_ctx, struct hns_roce_qp *qp,
 {
 	qp->dwqe_page = mmap(NULL, HNS_ROCE_DWQE_PAGE_SIZE, PROT_WRITE,
 			     MAP_SHARED, ibv_ctx->cmd_fd, dwqe_mmap_key);
-	if (qp->dwqe_page == MAP_FAILED)
+	if (qp->dwqe_page == MAP_FAILED) {
+		verbs_err(verbs_get_ctx(ibv_ctx),
+			  "failed to mmap direct wqe page, QPN = %u.\n",
+			  qp->verbs_qp.qp.qp_num);
 		return -EINVAL;
+	}
 
 	return 0;
 }
@@ -1336,6 +1571,7 @@ static struct ibv_qp *create_qp(struct ibv_context *ibv_ctx,
 				struct hnsdv_qp_init_attr *hns_attr)
 {
 	struct hns_roce_context *context = to_hr_ctx(ibv_ctx);
+	struct hns_roce_pad *pad = to_hr_pad(attr->pd);
 	struct hns_roce_qp *qp;
 	uint64_t dwqe_mmap_key;
 	int ret;
@@ -1352,6 +1588,13 @@ static struct ibv_qp *create_qp(struct ibv_context *ibv_ctx,
 
 	hns_roce_set_qp_params(attr, qp, context);
 
+	if (pad)
+		atomic_fetch_add(&pad->pd.refcount, 1);
+
+	ret = hns_roce_qp_spinlock_init(attr, qp);
+	if (ret)
+		goto err_spinlock;
+
 	ret = hns_roce_alloc_qp_buf(attr, qp, context);
 	if (ret)
 		goto err_buf;
@@ -1385,6 +1628,8 @@ err_ops:
 err_cmd:
 	hns_roce_free_qp_buf(qp, context);
 err_buf:
+	hns_roce_qp_spinlock_destroy(qp);
+err_spinlock:
 	free(qp);
 err:
 	if (ret < 0)
