@@ -141,9 +141,9 @@ static struct verbs_context *mlx4_alloc_context(struct ibv_device *ibdev,
 {
 	struct mlx4_context	       *context;
 	struct ibv_get_context		cmd;
-	struct mlx4_alloc_ucontext_resp resp;
+	struct mlx4_alloc_ucontext_resp resp = {};
 	int				i;
-	struct mlx4_alloc_ucontext_v3_resp resp_v3;
+	struct mlx4_alloc_ucontext_v3_resp resp_v3 = {};
 	__u16				bf_reg_size;
 	struct mlx4_device              *dev = to_mdev(ibdev);
 	struct verbs_context		*verbs_ctx;
