@@ -49,10 +49,11 @@ enum dr_dump_rec_type {
 	DR_DUMP_REC_TYPE_TABLE_TX = 3102,
 
 	DR_DUMP_REC_TYPE_MATCHER = 3200,
-	DR_DUMP_REC_TYPE_MATCHER_MASK = 3201,
+	DR_DUMP_REC_TYPE_MATCHER_MASK_DEPRECATED = 3201,
 	DR_DUMP_REC_TYPE_MATCHER_RX = 3202,
 	DR_DUMP_REC_TYPE_MATCHER_TX = 3203,
 	DR_DUMP_REC_TYPE_MATCHER_BUILDER = 3204,
+	DR_DUMP_REC_TYPE_MATCHER_MASK = 3205,
 
 	DR_DUMP_REC_TYPE_RULE = 3300,
 	DR_DUMP_REC_TYPE_RULE_RX_ENTRY_V0 = 3301,
@@ -81,6 +82,7 @@ enum dr_dump_rec_type {
 	DR_DUMP_REC_TYPE_ACTION_ASO_FLOW_METER = 3418,
 	DR_DUMP_REC_TYPE_ACTION_ASO_CT = 3419,
 	DR_DUMP_REC_TYPE_ACTION_MISS = 3423,
+	DR_DUMP_REC_TYPE_ACTION_ROOT_FT = 3424,
 };
 
 static uint64_t dr_dump_icm_to_idx(uint64_t icm_addr)
@@ -135,14 +137,41 @@ static int dr_dump_rule_action(FILE *f, const uint64_t rule_id,
 			      action->flow_tag);
 		break;
 	case DR_ACTION_TYP_MODIFY_HDR:
-		ret = fprintf(f, "%d,0x%" PRIx64 ",0x%" PRIx64 ",0x%x\n",
+	{
+		struct dr_ptrn_obj *ptrn = action->rewrite.ptrn_arg.ptrn;
+		struct dr_rewrite_param *param = &action->rewrite.param;
+		struct dr_arg_obj *arg = action->rewrite.ptrn_arg.arg;
+		bool ptrn_in_use = false;
+		int i;
+
+		if (!action->rewrite.single_action_opt && ptrn && arg)
+			ptrn_in_use = true;
+
+		ret = fprintf(f, "%d,0x%" PRIx64 ",0x%" PRIx64 ",0x%x,%d,0x%x,0x%" PRIx32 ",0x%" PRIx32,
 			      DR_DUMP_REC_TYPE_ACTION_MODIFY_HDR, action_id,
-			      rule_id, action->rewrite.index);
+			      rule_id, param->index,
+			      action->rewrite.single_action_opt,
+			      ptrn_in_use ? param->num_of_actions : 0,
+			      ptrn_in_use ? ptrn->rewrite_param.index : 0,
+			      ptrn_in_use ? dr_arg_get_object_id(arg) : 0);
+		if (ret < 0)
+			return ret;
+
+		if (ptrn_in_use) {
+			for (i = 0; i < param->num_of_actions; i++) {
+				ret = fprintf(f, ",0x%016" PRIx64,
+					      be64toh(((__be64 *)param->data)[i]));
+				if (ret < 0)
+					return ret;
+			}
+		}
+		ret = fprintf(f, "\n");
 		break;
+	}
 	case DR_ACTION_TYP_VPORT:
 		ret = fprintf(f, "%d,0x%" PRIx64 ",0x%" PRIx64 ",0x%x\n",
 			      DR_DUMP_REC_TYPE_ACTION_VPORT, action_id, rule_id,
-			      action->vport.num);
+			      action->vport.caps->num);
 		break;
 	case DR_ACTION_TYP_TNL_L2_TO_L2:
 		ret = fprintf(f, "%d,0x%" PRIx64 ",0x%" PRIx64 "\n",
@@ -152,17 +181,17 @@ static int dr_dump_rule_action(FILE *f, const uint64_t rule_id,
 	case DR_ACTION_TYP_TNL_L3_TO_L2:
 		ret = fprintf(f, "%d,0x%" PRIx64 ",0x%" PRIx64 ",0x%x\n",
 			      DR_DUMP_REC_TYPE_ACTION_DECAP_L3, action_id,
-			      rule_id, action->rewrite.index);
+			      rule_id, action->rewrite.param.index);
 		break;
 	case DR_ACTION_TYP_L2_TO_TNL_L2:
 		ret = fprintf(f, "%d,0x%" PRIx64 ",0x%" PRIx64 ",0x%x\n",
 			      DR_DUMP_REC_TYPE_ACTION_ENCAP_L2, action_id,
-			      rule_id, action->reformat.dvo->object_id);
+			      rule_id, dr_actions_reformat_get_id(action));
 		break;
 	case DR_ACTION_TYP_L2_TO_TNL_L3:
 		ret = fprintf(f, "%d,0x%" PRIx64 ",0x%" PRIx64 ",0x%x\n",
 			      DR_DUMP_REC_TYPE_ACTION_ENCAP_L3, action_id,
-			      rule_id, action->reformat.dvo->object_id);
+			      rule_id, dr_actions_reformat_get_id(action));
 		break;
 	case DR_ACTION_TYP_METER:
 		ret = fprintf(f, "%d,0x%" PRIx64 ",0x%" PRIx64 ",0x%" PRIx64 ",0x%x,0x%" PRIx64 ",0x%" PRIx64 "\n",
@@ -223,6 +252,12 @@ static int dr_dump_rule_action(FILE *f, const uint64_t rule_id,
 		ret = fprintf(f, "%d,0x%" PRIx64 ",0x%" PRIx64 "\n",
 			      DR_DUMP_REC_TYPE_ACTION_MISS, action_id, rule_id);
 		break;
+	case DR_ACTION_TYP_ROOT_FT:
+		ret = fprintf(f, "%d,0x%" PRIx64 ",0x%" PRIx64 ",0x%x\n",
+			      DR_DUMP_REC_TYPE_ACTION_ROOT_FT, action_id,
+			      rule_id,
+			      action->root_tbl.devx_tbl->ft_dvo->object_id);
+		break;
 	default:
 		return 0;
 	}
@@ -408,7 +443,7 @@ static int dr_dump_matcher_builder(FILE *f, struct dr_ste_build *builder,
 	bool is_match = builder->htbl_type == DR_STE_HTBL_TYPE_MATCH;
 	int ret;
 
-	ret = fprintf(f, "%d,0x%" PRIx64 "%d,%d,0x%x,%d\n",
+	ret = fprintf(f, "%d,0x%" PRIx64 ",%d,%d,0x%x,%d\n",
 		      DR_DUMP_REC_TYPE_MATCHER_BUILDER,
 		      matcher_id,
 		      index,
@@ -431,13 +466,14 @@ static int dr_dump_matcher_rx_tx(FILE *f, bool is_rx,
 	rec_type = is_rx ? DR_DUMP_REC_TYPE_MATCHER_RX :
 			   DR_DUMP_REC_TYPE_MATCHER_TX;
 
-	ret = fprintf(f, "%d,0x%" PRIx64 ",0x%" PRIx64 ",%d,0x%" PRIx64 ",0x%" PRIx64 "\n",
+	ret = fprintf(f, "%d,0x%" PRIx64 ",0x%" PRIx64 ",%d,0x%" PRIx64 ",0x%" PRIx64 ",%d\n",
 		      rec_type,
 		      (uint64_t) (uintptr_t) matcher_rx_tx,
 		      matcher_id,
 		      matcher_rx_tx->num_of_builders,
-		      dr_dump_icm_to_idx(matcher_rx_tx->s_htbl->chunk->icm_addr),
-		      dr_dump_icm_to_idx(matcher_rx_tx->e_anchor->chunk->icm_addr));
+		      dr_dump_icm_to_idx(dr_icm_pool_get_chunk_icm_addr(matcher_rx_tx->s_htbl->chunk)),
+		      dr_dump_icm_to_idx(dr_icm_pool_get_chunk_icm_addr(matcher_rx_tx->e_anchor->chunk)),
+		      matcher_rx_tx->fixed_size ? matcher_rx_tx->s_htbl->chunk_size : -1);
 	if (ret < 0)
 		return ret;
 
@@ -517,6 +553,7 @@ static int dr_dump_table_rx_tx(FILE *f, bool is_rx,
 			       struct dr_table_rx_tx *table_rx_tx,
 			       const uint64_t table_id)
 {
+	struct dr_icm_chunk *chunk = table_rx_tx->s_anchor->chunk;
 	enum dr_dump_rec_type rec_type;
 	int ret;
 
@@ -525,7 +562,7 @@ static int dr_dump_table_rx_tx(FILE *f, bool is_rx,
 	ret = fprintf(f, "%d,0x%" PRIx64 ",0x%" PRIx64 "\n",
 		      rec_type,
 		      table_id,
-		      dr_dump_icm_to_idx(table_rx_tx->s_anchor->chunk->icm_addr));
+		      dr_dump_icm_to_idx(dr_icm_pool_get_chunk_icm_addr(chunk)));
 	if (ret < 0)
 		return ret;
 
@@ -616,10 +653,39 @@ static int dr_dump_domain_info_flex_parser(FILE *f, const char *flex_parser_name
 	return 0;
 }
 
+static int dr_dump_vports_table(FILE *f, struct dr_vports_table *vports_tbl,
+				const uint64_t domain_id)
+{
+	struct dr_devx_vport_cap *vport_cap;
+	int i, ret;
+
+	if (!vports_tbl)
+		return 0;
+
+	for (i = 0; i < DR_VPORTS_BUCKETS; i++) {
+		vport_cap = vports_tbl->buckets[i];
+		while (vport_cap) {
+			ret = fprintf(f, "%d,0x%" PRIx64 ",%d,0x%x,0x%" PRIx64 ",0x%" PRIx64 "\n",
+				      DR_DUMP_REC_TYPE_DOMAIN_INFO_VPORT,
+				      domain_id,
+				      vport_cap->num,
+				      vport_cap->vport_gvmi,
+				      vport_cap->icm_address_rx,
+				      vport_cap->icm_address_tx);
+			if (ret < 0)
+				return ret;
+
+			vport_cap = vport_cap->next;
+		}
+	}
+
+	return 0;
+}
+
 static int dr_dump_domain_info_caps(FILE *f, struct dr_devx_caps *caps,
 					 const uint64_t domain_id)
 {
-	int i, ret;
+	int ret;
 
 	ret = fprintf(f, "%d,0x%" PRIx64 ",0x%x,0x%" PRIx64 ",0x%" PRIx64 ",0x%x,%d,%d\n",
 		      DR_DUMP_REC_TYPE_DOMAIN_INFO_CAPS,
@@ -628,22 +694,15 @@ static int dr_dump_domain_info_caps(FILE *f, struct dr_devx_caps *caps,
 		      caps->nic_rx_drop_address,
 		      caps->nic_tx_drop_address,
 		      caps->flex_protocols,
-		      caps->num_vports,
+		      caps->vports.num_ports,
 		      caps->eswitch_manager);
 	if (ret < 0)
 		return ret;
 
-	for (i = 0; i < caps->num_vports; i++) {
-		ret = fprintf(f, "%d,0x%" PRIx64 ",%d,0x%x,0x%" PRIx64 ",0x%" PRIx64 "\n",
-			      DR_DUMP_REC_TYPE_DOMAIN_INFO_VPORT,
-			      domain_id,
-			      i,
-			      caps->vports_caps[i].gvmi,
-			      caps->vports_caps[i].icm_address_rx,
-			      caps->vports_caps[i].icm_address_tx);
-		if (ret < 0)
-			return ret;
-	}
+	ret = dr_dump_vports_table(f, caps->vports.vports, domain_id);
+	if (ret < 0)
+		return ret;
+
 	return 0;
 }
 
@@ -652,11 +711,12 @@ static int dr_dump_domain_info_dev_attr(FILE *f, struct dr_domain_info *info,
 {
 	int ret;
 
-	ret = fprintf(f, "%d,0x%" PRIx64 ",%u,%s\n",
+	ret = fprintf(f, "%d,0x%" PRIx64 ",%u,%s,%d\n",
 		      DR_DUMP_REC_TYPE_DOMAIN_INFO_DEV_ATTR,
 		      domain_id,
-		      info->caps.num_vports + 1,
-		      info->attr.orig_attr.fw_ver);
+		      info->caps.vports.num_ports,
+		      info->attr.orig_attr.fw_ver,
+		      info->use_mqs);
 	if (ret < 0)
 		return ret;
 
@@ -699,18 +759,22 @@ static int dr_dump_domain(FILE *f, struct mlx5dv_dr_domain *dmn)
 	enum mlx5dv_dr_domain_type dmn_type = dmn->type;
 	char *dev_name = dmn->ctx->device->dev_name;
 	uint64_t domain_id;
-	int ret;
+	int ret, i;
 
 	domain_id = dr_domain_id_calc(dmn_type);
-
-	ret = fprintf(f, "%d,0x%" PRIx64 ",%d,0%x,%d,%s,%s\n",
+	ret = fprintf(f, "%d,0x%" PRIx64 ",%d,0%x,%d,%s,%s,%u,%u,%u,%u,%u\n",
 		      DR_DUMP_REC_TYPE_DOMAIN,
 		      domain_id,
 		      dmn_type,
 		      dmn->info.caps.gvmi,
 		      dmn->info.supp_sw_steering,
 		      PACKAGE_VERSION,
-		      dev_name);
+		      dev_name,
+		      dmn->flags,
+		      dmn->num_buddies[DR_ICM_TYPE_STE],
+		      dmn->num_buddies[DR_ICM_TYPE_MODIFY_ACTION],
+		      dmn->num_buddies[DR_ICM_TYPE_MODIFY_HDR_PTRN],
+		      dmn->info.caps.sw_format_ver);
 	if (ret < 0)
 		return ret;
 
@@ -719,9 +783,11 @@ static int dr_dump_domain(FILE *f, struct mlx5dv_dr_domain *dmn)
 		return ret;
 
 	if (dmn->info.supp_sw_steering) {
-		ret = dr_dump_send_ring(f, dmn->send_ring, domain_id);
-		if (ret < 0)
-			return ret;
+		for (i = 0; i < DR_MAX_SEND_RINGS; i++) {
+			ret = dr_dump_send_ring(f, dmn->send_ring[i], domain_id);
+			if (ret < 0)
+				return ret;
+		}
 	}
 
 	return 0;
@@ -752,11 +818,13 @@ int mlx5dv_dump_dr_domain(FILE *fout, struct mlx5dv_dr_domain *dmn)
 	if (!fout || !dmn)
 		return -EINVAL;
 
+	pthread_spin_lock(&dmn->debug_lock);
 	dr_domain_lock(dmn);
 
 	ret = dr_dump_domain_all(fout, dmn);
 
 	dr_domain_unlock(dmn);
+	pthread_spin_unlock(&dmn->debug_lock);
 
 	return ret;
 }
@@ -768,7 +836,9 @@ int mlx5dv_dump_dr_table(FILE *fout, struct mlx5dv_dr_table *tbl)
 	if (!fout || !tbl)
 		return -EINVAL;
 
+	pthread_spin_lock(&tbl->dmn->debug_lock);
 	dr_domain_lock(tbl->dmn);
+
 	ret = dr_dump_domain(fout, tbl->dmn);
 	if (ret < 0)
 		goto out;
@@ -776,6 +846,7 @@ int mlx5dv_dump_dr_table(FILE *fout, struct mlx5dv_dr_table *tbl)
 	ret = dr_dump_table_all(fout, tbl);
 out:
 	dr_domain_unlock(tbl->dmn);
+	pthread_spin_unlock(&tbl->dmn->debug_lock);
 	return ret;
 }
 
@@ -786,7 +857,9 @@ int mlx5dv_dump_dr_matcher(FILE *fout, struct mlx5dv_dr_matcher *matcher)
 	if (!fout || !matcher)
 		return -EINVAL;
 
+	pthread_spin_lock(&matcher->tbl->dmn->debug_lock);
 	dr_domain_lock(matcher->tbl->dmn);
+
 	ret = dr_dump_domain(fout, matcher->tbl->dmn);
 	if (ret < 0)
 		goto out;
@@ -798,6 +871,7 @@ int mlx5dv_dump_dr_matcher(FILE *fout, struct mlx5dv_dr_matcher *matcher)
 	ret = dr_dump_matcher_all(fout, matcher);
 out:
 	dr_domain_unlock(matcher->tbl->dmn);
+	pthread_spin_unlock(&matcher->tbl->dmn->debug_lock);
 	return ret;
 }
 
@@ -808,7 +882,9 @@ int mlx5dv_dump_dr_rule(FILE *fout, struct mlx5dv_dr_rule *rule)
 	if (!fout || !rule)
 		return -EINVAL;
 
+	pthread_spin_lock(&rule->matcher->tbl->dmn->debug_lock);
 	dr_domain_lock(rule->matcher->tbl->dmn);
+
 	ret = dr_dump_domain(fout, rule->matcher->tbl->dmn);
 	if (ret < 0)
 		goto out;
@@ -824,6 +900,7 @@ int mlx5dv_dump_dr_rule(FILE *fout, struct mlx5dv_dr_rule *rule)
 	ret = dr_dump_rule(fout, rule);
 out:
 	dr_domain_unlock(rule->matcher->tbl->dmn);
+	pthread_spin_unlock(&rule->matcher->tbl->dmn->debug_lock);
 	return ret;
 }
 
