@@ -68,7 +68,7 @@ static const struct verbs_match_ent ucna_table[] = {
 };
 
 static const struct verbs_context_ops ocrdma_ctx_ops = {
-	.query_device = ocrdma_query_device,
+	.query_device_ex = ocrdma_query_device,
 	.query_port = ocrdma_query_port,
 	.alloc_pd = ocrdma_alloc_pd,
 	.dealloc_pd = ocrdma_free_pd,
@@ -115,7 +115,7 @@ static struct verbs_context *ocrdma_alloc_context(struct ibv_device *ibdev,
 {
 	struct ocrdma_devctx *ctx;
 	struct uocrdma_get_context cmd;
-	struct uocrdma_get_context_resp resp;
+	struct uocrdma_get_context_resp resp = {};
 
 	ctx = verbs_init_and_alloc_context(ibdev, cmd_fd, ctx, ibv_ctx,
 					   RDMA_DRIVER_OCRDMA);
