@@ -68,17 +68,21 @@ static inline void ocrdma_swap_cpu_to_le(void *dst, uint32_t len)
  * ocrdma_query_device
  */
 int ocrdma_query_device(struct ibv_context *context,
-			struct ibv_device_attr *attr)
+			const struct ibv_query_device_ex_input *input,
+			struct ibv_device_attr_ex *attr, size_t attr_size)
 {
-	struct ibv_query_device cmd;
-	uint64_t fw_ver;
 	struct ocrdma_device *dev = get_ocrdma_dev(context->device);
-	int status;
+	struct ib_uverbs_ex_query_device_resp resp;
+	size_t resp_size = sizeof(resp);
+	int ret;
 
-	bzero(attr, sizeof *attr);
-	status = ibv_cmd_query_device(context, attr, &fw_ver, &cmd, sizeof cmd);
-	memcpy(attr->fw_ver, dev->fw_ver, sizeof(dev->fw_ver));
-	return status;
+	ret = ibv_cmd_query_device_any(context, input, attr, attr_size, &resp,
+				       &resp_size);
+	if (ret)
+		return ret;
+
+	memcpy(attr->orig_attr.fw_ver, dev->fw_ver, sizeof(dev->fw_ver));
+	return 0;
 }
 
 /*
@@ -185,22 +189,20 @@ int ocrdma_free_pd(struct ibv_pd *ibpd)
 /*
  * ocrdma_reg_mr
  */
-struct ibv_mr *ocrdma_reg_mr(struct ibv_pd *pd, void *addr,
-			     size_t len, int access)
+struct ibv_mr *ocrdma_reg_mr(struct ibv_pd *pd, void *addr, size_t len,
+			     uint64_t hca_va, int access)
 {
 	struct ocrdma_mr *mr;
 	struct ibv_reg_mr cmd;
 	struct uocrdma_reg_mr_resp resp;
-	uint64_t hca_va = (uintptr_t) addr;
 
 	mr = malloc(sizeof *mr);
 	if (!mr)
 		return NULL;
 	bzero(mr, sizeof *mr);
 
-	if (ibv_cmd_reg_mr(pd, addr, len, hca_va,
-			   access, &mr->vmr, &cmd, sizeof(cmd),
-			   &resp.ibv_resp, sizeof(resp))) {
+	if (ibv_cmd_reg_mr(pd, addr, len, hca_va, access, &mr->vmr, &cmd,
+			   sizeof(cmd), &resp.ibv_resp, sizeof(resp))) {
 		free(mr);
 		return NULL;
 	}
@@ -354,7 +356,7 @@ struct ibv_srq *ocrdma_create_srq(struct ibv_pd *pd,
 	int status = 0;
 	struct ocrdma_srq *srq;
 	struct uocrdma_create_srq cmd;
-	struct uocrdma_create_srq_resp resp;
+	struct uocrdma_create_srq_resp resp = {};
 	void *map_addr;
 
 	srq = calloc(1, sizeof *srq);
@@ -464,7 +466,7 @@ struct ibv_qp *ocrdma_create_qp(struct ibv_pd *pd,
 {
 	int status = 0;
 	struct uocrdma_create_qp cmd;
-	struct uocrdma_create_qp_resp resp;
+	struct uocrdma_create_qp_resp resp = {};
 	struct ocrdma_qp *qp;
 	void *map_addr;
 #ifdef DPP_CQ_SUPPORT
@@ -979,7 +981,6 @@ static inline void ocrdma_srq_inc_tail(struct ocrdma_qp *qp,
 
 static void ocrdma_discard_cqes(struct ocrdma_qp *qp, struct ocrdma_cq *cq)
 {
-	int discard_cnt = 0;
 	uint32_t cur_getp, stop_getp;
 	struct ocrdma_cqe *cqe;
 	uint32_t qpn = 0;
@@ -1027,7 +1028,6 @@ static void ocrdma_discard_cqes(struct ocrdma_qp *qp, struct ocrdma_cq *cq)
 				ocrdma_hwq_inc_tail(&qp->rq);
 		}
 
-		discard_cnt += 1;
 		/* discard by marking qp_id = 0 */
 		cqe->cmn.qpn = 0;
 skip_cqe:
